"""Version information for waldiez_studio.

Dev: Do not edit this file.
Use ../package.json to update the version instead.
The version will be updated automatically by Hatchling.
See ../scripts/hook.py for more information.
"""

__version__ = "0.2.1"
